@@ -1,35 +1,36 @@
 ;; raise-commands.jl -- some commands for raising windows
-;; $Id: raise-commands.jl,v 1.11 2002/04/21 04:52:14 jsh Exp $
 
 ;; Copyright (C) 1999-2000 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawmill.
+;; This file is part of sawfish.
 
-;; sawmill is free software; you can redistribute it and/or modify it
+;; sawfish is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawmill is distributed in the hope that it will be useful, but
+;; sawfish is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawmill; see the file COPYING.  If not, write to
+;; along with sawfish; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(define-structure sawfish.wm.commands.raise-commands ()
+(define-structure sawfish.wm.commands.raise-commands
+
+    (export )
 
     (open rep
-	  sawfish.wm.windows
-	  sawfish.wm.events
-	  sawfish.wm.stacking
-	  sawfish.wm.util.stacking
-	  sawfish.wm.state.transient
-	  sawfish.wm.commands.groups
-	  sawfish.wm.commands
-	  sawfish.wm.focus)
+          sawfish.wm.windows
+          sawfish.wm.events
+          sawfish.wm.stacking
+          sawfish.wm.util.stacking
+          sawfish.wm.state.transient
+          sawfish.wm.commands.groups
+          sawfish.wm.commands
+          sawfish.wm.focus)
 
   (define (replay-pointer w)
     ;; click-to-focus mode sets this to t when it calls a command
