@@ -1,22 +1,21 @@
 ;; place-window.jl -- decide where to initially place a window
-;; $Id: placement.jl,v 1.55 2003/01/12 20:48:36 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawmill.
+;; This file is part of sawfish.
 
-;; sawmill is free software; you can redistribute it and/or modify it
+;; sawfish is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawmill is distributed in the hope that it will be useful, but
+;; sawfish is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawmill; see the file COPYING.  If not, write to
+;; along with sawfish; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.placement
@@ -40,20 +39,21 @@
   (defcustom place-window-mode 'top-left
     "Method of placing windows: \\w"
     :type symbol
-    :group placement)
+    :group (misc placement))
 
   (defcustom place-transient-mode 'centered-on-parent
     "Method of placing dialog windows: \\w"
     :type symbol
-    :group placement)
+    :group (misc placement))
 
-  (defvar ignore-program-positions t
-    "Ignore program-specified window placements.")
+  (defcustom ignore-program-positions t
+    "Ignore program-specified window placements."
+    :type boolean
+    :group (misc placement))
 
   (defvar placement-modes '()
     "List of names of all placement modes.")
 
-
 ;;; utility functions
 
   ;; autoload handling
@@ -128,7 +128,6 @@ this mode. The single argument is the window to be placed."
   (add-hook 'place-window-hook place-window t)
   (add-hook 'remove-window-hook unplace-window)
 
-
 ;;; standard placement modes
 
   (define (place-window-randomly w)
@@ -188,7 +187,8 @@ this mode. The single argument is the window to be placed."
 	(let ((dims (window-frame-dimensions w))
 	      (pdims (window-frame-dimensions parent))
 	      (coords (window-position parent))
-	      (screen (calculate-workarea #:window w #:head (current-head parent))))
+	      (screen (calculate-workarea #:window w
+                                          #:head (current-head parent))))
 	  (rplaca coords (clamp* (+ (car coords)
 				    (quotient (- (car pdims) (car dims)) 2))
 				 (car dims) (nth 0 screen) (nth 2 screen)))
@@ -207,9 +207,15 @@ this mode. The single argument is the window to be placed."
 			     (cdr dims) (nth 1 screen) (nth 3 screen)))
       (move-window-to w (car coords) (cdr coords))))
 
-  (define-placement-mode 'randomly place-window-randomly #:for-normal t #:for-dialogs t)
-  (define-placement-mode 'interactively place-window-interactively #:for-normal t)
-  (define-placement-mode 'centered place-window-centered #:for-normal t #:for-dialogs t)
-  (define-placement-mode 'centered-on-parent place-window-centered-on-parent #:for-dialogs t)
-  (define-placement-mode 'under-pointer place-window-under-pointer #:for-dialogs t)
-  (define-placement-mode 'none nop #:for-normal t #:for-dialogs t))
+  (define-placement-mode 'randomly place-window-randomly
+    #:for-normal t #:for-dialogs t)
+  (define-placement-mode 'interactively place-window-interactively
+    #:for-normal t #:for-dialogs t)
+  (define-placement-mode 'centered place-window-centered
+    #:for-normal t #:for-dialogs t)
+  (define-placement-mode 'centered-on-parent place-window-centered-on-parent
+    #:for-normal t #:for-dialogs t)
+  (define-placement-mode 'under-pointer place-window-under-pointer
+    #:for-normal t #:for-dialogs t)
+  (define-placement-mode 'none nop
+    #:for-normal t #:for-dialogs t))
