@@ -1,22 +1,21 @@
 ;; sm-init.jl -- session manager code loaded on startup
-;; $Id: init.jl,v 1.20 2002/11/04 03:35:39 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawmill.
+;; This file is part of sawfish.
 
-;; sawmill is free software; you can redistribute it and/or modify it
+;; sawfish is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawmill is distributed in the hope that it will be useful, but
+;; sawfish is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawmill; see the file COPYING.  If not, write to
+;; along with sawfish; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.session.init
@@ -38,10 +37,10 @@
 	  sawfish.wm.session.util)
 
   (define sm-client-id nil
-    "A string identifying the current session.")
+          "A string identifying the current session.")
 
   (define sm-prefix nil
-    "A string used to uniquify the session file.")
+          "A string used to uniquify the session file.")
 
   ;; this is before both the panel and gmc..?
   (defconst sm-gsm-priority 30)
@@ -104,7 +103,8 @@ that feature off, allowing some broken clients to be session managed.")
 	       (leader (cond ((and (setq tem (get-x-property
 					      w 'WM_CLIENT_LEADER))
 				   (eq (car tem) 'WINDOW)
-				   (eq (nth 1 tem) 32))
+				   (eq (nth 1 tem) 32)
+				   (not (zerop (aref (nth 2 tem) 0))))
 			      (aref (nth 2 tem) 0))
 			     ((window-group-id w))
 			     ((window-transient-p w)))))
@@ -112,7 +112,7 @@ that feature off, allowing some broken clients to be session managed.")
 
   (define (make-unique-prefix) (number->string (current-utime) 36))
 
-;; callback
+  ;; callback
 
   (define (sm-save-yourself)
     (require 'sawfish.wm.session.save)
@@ -143,14 +143,15 @@ that feature off, allowing some broken clients to be session managed.")
   (define (remove-sm-options)
     ;; remove any sm options from saved-command-line-args
     (let loop ((args saved-command-line-args))
-      (when (cdr args)
-	(if (string-match "^(--sm-client-id|-clientId|--sm-prefix)" (cadr args))
-	    (progn
-	      (if (string-match "=" (cadr args))
-		  (rplacd args (cddr args))
-		(rplacd args (cdddr args)))
-	      (loop args))
-	  (loop (cdr args))))))
+         (when (cdr args)
+           (if (string-match "^(--sm-client-id|-clientId|--sm-prefix)"
+                             (cadr args))
+               (progn
+                 (if (string-match "=" (cadr args))
+                     (rplacd args (cddr args))
+                   (rplacd args (cdddr args)))
+                 (loop args))
+             (loop (cdr args))))))
 
   (define (set-discard-command)
     (sm-set-property
@@ -175,7 +176,7 @@ that feature off, allowing some broken clients to be session managed.")
 
       ;; XXX should I set this to SmRestartImmediately (2) instead
       ;; XXX of SmRestartIfRunning (0) ?
-      (sm-set-property "RestartStyleHint" 0)
+      (sm-set-property "RestartStyleHint" 2)
 
       (remove-sm-options)
       (sm-set-property "CloneCommand" saved-command-line-args)
