@@ -1,23 +1,21 @@
 ;; wm-spec.jl -- implement the new (GNOME/KDE) wm hints spec
 
-;; $Id: wm-spec.jl,v 1.46 2003/03/16 20:56:48 jsh Exp $
-
 ;; Copyright (C) 1999, 2000 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawmill.
+;; This file is part of sawfish.
 
-;; sawmill is free software; you can redistribute it and/or modify it
+;; sawfish is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawmill is distributed in the hope that it will be useful, but
+;; sawfish is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawmill; see the file COPYING.  If not, write to
+;; along with sawfish; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.state.wm-spec
@@ -50,7 +48,6 @@
   ;; cover but existed in the old GNOME spec; e.g. _GNOME_WM_STATE_FOO
   ;; for FOO being DO_NOT_COVER, SKIP_FOCUS, ..?
 
-
 ;;; constants
 
   (defconst _NET_WM_MOVERESIZE_SIZE_TOPLEFT 0)
@@ -123,33 +120,32 @@
      _NET_WM_WINDOW_TYPE_TOOLBAR
      _NET_WM_WINDOW_TYPE_MENU
      _NET_WM_WINDOW_TYPE_UTILITY
-     _NET_WM_WINDOW_TYPE_SPLASH])
-  
-  (defvar wm-spec-below-depth +2)
+     _NET_WM_WINDOW_TYPE_SPLASH
+     _NET_WM_USER_TIME])
+
+  (defvar wm-spec-below-depth -2)
   (defvar wm-spec-above-depth +2)
 
   (define supported-states '())
 
-
 ;;; setting the client list hints
 
   (define (update-client-list-hints #!key only-stacking-list)
     (define (set-prop lst prop)
       (let loop ((rest lst)
 		 (collected '()))
-	(cond ((null rest)
-	       (set-x-property 'root prop
-			       (apply vector (nreverse collected))
-			       'WINDOW 32))
-	      ((window-mapped-p (car rest))
-	       (loop (cdr rest) (cons (window-id (car rest)) collected)))
-	      (t (loop (cdr rest) collected)))))
+           (cond ((null rest)
+                  (set-x-property 'root prop
+                                  (apply vector (nreverse collected))
+                                  'WINDOW 32))
+                 ((window-mapped-p (car rest))
+                  (loop (cdr rest) (cons (window-id (car rest)) collected)))
+                 (t (loop (cdr rest) collected)))))
     (unless only-stacking-list
       (set-prop (managed-windows) '_NET_CLIENT_LIST))
     (set-prop (nreverse (stacking-order)) '_NET_CLIENT_LIST_STACKING))
 
-
-;; setting the desktop / viewport hints
+  ;; setting the desktop / viewport hints
 
   (define last-workspace nil)
   (define last-workspace-count 0)
@@ -159,10 +155,31 @@
   (define last-workarea nil)
   (define last-showing-desktop nil)
 
+  (define (update-window-workspace-hints w #!key (limits (workspace-limits)))
+    (let ((vec (if (window-sticky-p/workspace w)
+		   (vector #xffffffff)
+		 (let ((space (or (window-get w 'swapped-in)
+				  (car (window-workspaces w)))))
+		   (and space (vector (- space (car limits))))))))
+      (unless (equal vec (window-get w 'wm-spec/last-workspace))
+	(if vec
+	    (set-x-property w '_NET_WM_DESKTOP vec 'CARDINAL 32)
+	  (delete-x-property w '_NET_WM_DESTOP))
+	(window-put w 'wm-spec/last-workspace vec))))
+
   (define (update-workspace-hints)
     (let* ((limits (workspace-limits))
 	   (port (screen-viewport))
-	   (port-size viewport-dimensions)
+           ;; Since vp size can vary from workspace to workspace, use
+           ;; the maximum dimensions accross all workspaces.
+	   (port-size (let ((dims (cons viewport-dimensions
+                                        (mapcar (lambda (e)
+                                                  (unless (eq (car e)
+                                                              current-workspace)
+                                                    (nth 3 e)))
+                                                workspace-viewport-data))))
+                        (cons (apply max (mapcar car dims))
+                              (apply max (mapcar cdr dims)))))
 	   (total-workspaces (1+ (- (cdr limits) (car limits))))
 	   (workarea (make-vector (* 4 total-workspaces)))
 	   (showing-desktop (showing-desktop-p)))
@@ -200,12 +217,18 @@
 	(unless (equal last-area port)
 	  (let ((view (make-vector (* total-workspaces 2))))
 	    (let loop ((i 0))
-	      (if (= i total-workspaces)
-		  (set-x-property 'root '_NET_DESKTOP_VIEWPORT
-				  view 'CARDINAL 32)
-		(aset view (* i 2) (* (car port) (screen-width)))
-		(aset view (1+ (* i 2)) (* (cdr port) (screen-height)))
-		(loop (1+ i))))))
+                 (if (= i total-workspaces)
+                     (set-x-property 'root '_NET_DESKTOP_VIEWPORT
+                                     view 'CARDINAL 32)
+                   (if (eq i current-workspace)
+                       (progn
+                         (aset view (* i 2) (* (car port) (screen-width)))
+                         (aset view (1+ (* i 2)) (* (cdr port)
+                                                    (screen-height))))
+                     (let ((vp-data (cdr (assoc i workspace-viewport-data))))
+                       (aset view (* i 2) (car vp-data))
+                       (aset view (1+ (* i 2)) (nth 1 vp-data))))
+                   (loop (1+ i))))))
 
 	;; _NET_WORKAREA
 	(unless (equal last-workarea workarea)
@@ -219,16 +242,7 @@
 	  (setq last-showing-desktop showing-desktop)))
 
       (define (set-window-hints w)
-	(let ((vec (if (window-sticky-p/workspace w)
-		       (vector #xffffffff)
-		     (let ((space (or (window-get w 'swapped-in)
-				      (car (window-workspaces w)))))
-		       (and space (vector (- space (car limits))))))))
-	  (unless (equal vec (window-get w 'wm-spec/last-workspace))
-	    (if vec
-		(set-x-property w '_NET_WM_DESKTOP vec 'CARDINAL 32)
-	      (delete-x-property w '_NET_WM_DESTOP))
-	    (window-put w 'wm-spec/last-workspace vec))))
+	(update-window-workspace-hints w #:limits limits))
 
       ;; calculate workareas
       (do ((i 0 (1+ i)))
@@ -239,7 +253,7 @@
 	  (aset workarea (+ (* i 4) 1) (nth 1 area))
 	  (aset workarea (+ (* i 4) 2) (- (nth 2 area) (nth 0 area)))
 	  (aset workarea (+ (* i 4) 3) (- (nth 3 area) (nth 1 area)))))
-		 
+
       ;; apparently some pagers don't like it if we place windows
       ;; on (temporarily) non-existent workspaces
       (when (< last-workspace-count total-workspaces)
@@ -250,7 +264,6 @@
       (when (>= last-workspace-count total-workspaces)
 	(set-ws-hints))))
 
-
 ;;; setting the focus hints
 
   (define last-focus nil)
@@ -262,7 +275,6 @@
 	(set-x-property 'root '_NET_ACTIVE_WINDOW
 			(vector (if focus (window-id focus) 0)) 'WINDOW 32))))
 
-
 ;;; setting the window state hints
 
   (define (update-client-state w)
@@ -274,7 +286,6 @@
 	    supported-states)
       (set-x-property w '_NET_WM_STATE (apply vector state) 'ATOM 32)))
 
-
 ;;; honouring the initially set window state hints
 
   (define (update-icon-geometry w geom)
@@ -286,7 +297,7 @@
       (when (and strut (eq (nth 0 strut) 'CARDINAL))
 	(let ((data (nth 2 strut)))
 	  (define-window-strut w (aref data 0) (aref data 2)
-			       (aref data 1) (aref data 3))))))
+            (aref data 1) (aref data 3))))))
 
   (define (honour-client-state w)
     (let ((space (get-x-property w '_NET_WM_DESKTOP)))
@@ -303,10 +314,10 @@
 	;; _NET_WM_WINDOW_TYPE is a vector of atoms, the first atom
 	;; about which we know something is the type we'll use
 	(let loop ((i 0))
-	  (cond ((= i (length type)))
-		((get (aref type i) 'wm-spec-type)
-		 ((get (aref type i) 'wm-spec-type) w))
-		(t (loop (1+ i)))))))
+             (cond ((= i (length type)))
+                   ((get (aref type i) 'wm-spec-type)
+                    ((get (aref type i) 'wm-spec-type) w))
+                   (t (loop (1+ i)))))))
 
     (let ((state (get-x-property w '_NET_WM_STATE)))
       (when state
@@ -319,9 +330,11 @@
 
     (let ((geom (get-x-property w '_NET_WM_ICON_GEOMETRY)))
       (when geom
-	(update-icon-geometry w (nth 2 geom)))))
+	(update-icon-geometry w (nth 2 geom))))
+    (when (equal (get-x-property w '_NET_WM_USER_TIME)
+                 '(CARDINAL 32 #(0)))
+      (window-put w 'inhibit-focus-when-mapped t)))
 
-
 ;;; helper functions
 
   (define (define-wm-spec-window-type x fun)
@@ -345,48 +358,48 @@
 	(fun w mode))))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DESKTOP
-   (lambda (w)
-     (mark-window-as-desktop w)))
+    (lambda (w)
+      (mark-window-as-desktop w)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DOCK
-   (lambda (w)
-     (mark-window-as-dock w)))
+    (lambda (w)
+      (mark-window-as-dock w)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DIALOG
-   (lambda (w)
-     (mark-window-as-transient w)))
+    (lambda (w)
+      (mark-window-as-transient w)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_UTILITY
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'utility)))
+    (lambda (w)
+      (require 'sawfish.wm.frames)
+      (set-window-type w 'utility)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_TOOLBAR
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'toolbar)))
+    (lambda (w)
+      (require 'sawfish.wm.frames)
+      (set-window-type w 'toolbar)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_MENU
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'menu)))
+    (lambda (w)
+      (require 'sawfish.wm.frames)
+      (set-window-type w 'menu)))
 
   (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_SPLASH
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'splash)
-     (window-put w 'place-mode 'centered)))
+    (lambda (w)
+      (require 'sawfish.wm.frames)
+      (set-window-type w 'splash)
+      (window-put w 'place-mode 'centered)))
 
   (define-wm-spec-window-state '_NET_WM_STATE_STICKY
-   (lambda (w mode)
-     (case mode
-       ((init)   (window-put w 'sticky-viewport t))
-       ((remove) (make-window-unsticky/viewport w))
-       ((add)    (make-window-sticky/viewport w))
-       ((toggle) (if (window-sticky-p/viewport w)
-		     (make-window-unsticky/viewport w)
-		   (make-window-sticky/viewport w)))
-       ((get)    (window-sticky-p/viewport w)))))
+    (lambda (w mode)
+      (case mode
+        ((init)   (window-put w 'sticky-viewport t))
+        ((remove) (make-window-unsticky/viewport w))
+        ((add)    (make-window-sticky/viewport w))
+        ((toggle) (if (window-sticky-p/viewport w)
+                      (make-window-unsticky/viewport w)
+                    (make-window-sticky/viewport w)))
+        ((get)    (window-sticky-p/viewport w)))))
 
   (define (wm-spec-maximize-handler direction)
     (lambda (w mode)
@@ -407,49 +420,49 @@
 		      (t (window-maximized-p w))))))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_MAXIMIZED_VERT
-			       (wm-spec-maximize-handler 'vertical))
+    (wm-spec-maximize-handler 'vertical))
   (define-wm-spec-window-state '_NET_WM_STATE_MAXIMIZED_HORZ
-			       (wm-spec-maximize-handler 'horizontal))
+    (wm-spec-maximize-handler 'horizontal))
   (define-wm-spec-window-state '_NET_WM_STATE_MAXIMIZED
-			       (wm-spec-maximize-handler nil)
-			       #:pseudo t)
+    (wm-spec-maximize-handler nil)
+    #:pseudo t)
 
   (define-wm-spec-window-state '_NET_WM_STATE_SHADED
-   (lambda (w mode)
-     (require 'sawfish.wm.state.shading)
-     (case mode
-       ((init)   (window-put w 'shaded t))
-       ((add)    (shade-window w))
-       ((remove) (unshade-window w))
-       ((toggle) (toggle-window-shaded w))
-       ((get)    (window-get w 'shaded)))))
+    (lambda (w mode)
+      (require 'sawfish.wm.state.shading)
+      (case mode
+        ((init)   (window-put w 'shaded t))
+        ((add)    (shade-window w))
+        ((remove) (unshade-window w))
+        ((toggle) (toggle-window-shaded w))
+        ((get)    (window-get w 'shaded)))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_SKIP_PAGER
-   (lambda (w mode)
-     (case mode
-       ((init add) (window-put w 'window-list-skip t))
-       ((remove)   (window-put w 'window-list-skip nil))
-       ((toggle)   (window-put w 'window-list-skip
-			       (not (window-get w 'window-list-skip))))
-       ((get)      (window-get w 'window-list-skip)))))
+    (lambda (w mode)
+      (case mode
+        ((init add) (window-put w 'window-list-skip t))
+        ((remove)   (window-put w 'window-list-skip nil))
+        ((toggle)   (window-put w 'window-list-skip
+                                (not (window-get w 'window-list-skip))))
+        ((get)      (window-get w 'window-list-skip)))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_SKIP_TASKBAR
-   (lambda (w mode)
-     (case mode
-       ((init add) (window-put w 'task-list-skip t))
-       ((remove)   (window-put w 'task-list-skip nil))
-       ((toggle)   (window-put w 'task-list-skip
-			       (not (window-get w 'task-list-skip))))
-       ((get)      (window-get w 'task-list-skip)))))
+    (lambda (w mode)
+      (case mode
+        ((init add) (window-put w 'task-list-skip t))
+        ((remove)   (window-put w 'task-list-skip nil))
+        ((toggle)   (window-put w 'task-list-skip
+                                (not (window-get w 'task-list-skip))))
+        ((get)      (window-get w 'task-list-skip)))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_FULLSCREEN
-   (lambda (w mode)
-     (require 'sawfish.wm.state.maximize)
-     (case mode
-       ((init) (window-put w 'queued-fullscreen-maximize t))
-       ((add remove) (maximize-window-fullscreen w (eq mode 'add)))
-       ((toggle) (maximize-window-fullscreen-toggle w))
-       ((get) (window-maximized-fullscreen-p w)))))
+    (lambda (w mode)
+      (require 'sawfish.wm.state.maximize)
+      (case mode
+        ((init) (window-put w 'queued-fullscreen-maximize t))
+        ((add remove) (maximize-window-fullscreen w (eq mode 'add)))
+        ((toggle) (maximize-window-fullscreen-toggle w))
+        ((get) (window-maximized-fullscreen-p w)))))
 
   (define (above-below-handler depth w mode)
     (require 'sawfish.wm.stacking)
@@ -464,14 +477,13 @@
        (= (window-depth w) depth))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_BELOW
-   (lambda (w mode)
-     (above-below-handler wm-spec-below-depth w mode)))
+    (lambda (w mode)
+      (above-below-handler wm-spec-below-depth w mode)))
 
   (define-wm-spec-window-state '_NET_WM_STATE_ABOVE
-   (lambda (w mode)
-     (above-below-handler wm-spec-above-depth w mode)))
+    (lambda (w mode)
+      (above-below-handler wm-spec-above-depth w mode)))
 
-
 ;;; client messages
 
   (define (client-message-handler w type data)
@@ -496,14 +508,22 @@
 		     (eq mode _NET_WM_MOVERESIZE_MOVE_KEYBOARD))
 		 (move-window-interactively w)
 	       (let ((move-resize-moving-edges
-		      (cond ((eq mode _NET_WM_MOVERESIZE_SIZE_TOPLEFT) '(top left))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_TOP) '(top))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_TOPRIGHT) '(top right))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT) '(bottom left))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOM) '(bottom))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT) '(bottom right))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_LEFT) '(left))
-			    ((eq mode _NET_WM_MOVERESIZE_SIZE_RIGHT) '(right)))))
+		      (cond ((eq mode _NET_WM_MOVERESIZE_SIZE_TOPLEFT)
+                             '(top left))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_TOP)
+                             '(top))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_TOPRIGHT)
+                             '(top right))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT)
+                             '(bottom left))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOM)
+                             '(bottom))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT)
+                             '(bottom right))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_LEFT)
+                             '(left))
+			    ((eq mode _NET_WM_MOVERESIZE_SIZE_RIGHT)
+                             '(right)))))
 		 (resize-window-interactively w))))))
 
 	((_NET_NUMBER_OF_DESKTOPS)
@@ -516,7 +536,12 @@
 	 (set-viewport (aref data 0) (aref data 1)))
 
 	((_NET_CURRENT_DESKTOP)
-	 (select-workspace (workspace-id-from-logical (aref data 0))))
+         ;; KDE spews _NET_CURRENT_DESKTOP( -1) messages so often that it
+         ;; is best to just ignore out of bounds errors silently.
+         (let ((ws (workspace-id-from-logical (aref data 0)))
+               (limits (workspace-limits)))
+           (if (<= (car limits) ws (cdr limits))
+               (select-workspace ws))))
 
 	((_NET_DESKTOP_NAMES)
 	 ;; XXX this is kind of broken now we use workspace-names to
@@ -524,9 +549,9 @@
 	 (setq data (aref data 0))
 	 (let loop ((i 0)
 		    (out '()))
-	   (if (= i (length data))
-	       (setq workspace-names (nreverse out))
-	     (loop (1+ i) (cons (aref data i) out)))))
+              (if (= i (length data))
+                  (setq workspace-names (nreverse out))
+                (loop (1+ i) (cons (aref data i) out)))))
 
 	((_NET_ACTIVE_WINDOW)
 	 (require 'sawfish.wm.util.display-window)
@@ -535,9 +560,12 @@
 
 	((_NET_WM_STATE)
 	 (when (windowp w)
-	   (let ((mode (cond ((eql (aref data 0) _NET_WM_STATE_REMOVE) 'remove)
-			     ((eql (aref data 0) _NET_WM_STATE_ADD) 'add)
-			     ((eql (aref data 0) _NET_WM_STATE_TOGGLE) 'toggle)))
+	   (let ((mode (cond ((eql (aref data 0) _NET_WM_STATE_REMOVE)
+                              'remove)
+			     ((eql (aref data 0) _NET_WM_STATE_ADD)
+                              'add)
+			     ((eql (aref data 0) _NET_WM_STATE_TOGGLE)
+                              'toggle)))
 		 (atom1 (x-atom-name (aref data 1)))
 		 (atom2 (x-atom-name (aref data 2))))
 	     (when (or (and (eq atom1 '_NET_WM_STATE_MAXIMIZED_VERT)
@@ -564,7 +592,6 @@
 	(t (setq handled nil)))
       handled))
 
-
 ;;; property changes
 
   (define (property-change-handler w prop kind)
@@ -577,7 +604,6 @@
       ((_NET_WM_STRUT)
        (update-strut w))))
 
-
 ;;; utilities
 
   (define (vector->list vec)
@@ -585,7 +611,10 @@
 	 (out '() (cons (aref vec i) out)))
 	((= i (length vec)) (nreverse out))))
 
-
+  (define (update-on-configure-notify w)
+    (when (eq w 'root)
+      (update-workspace-hints)))
+
 ;;; initialisation
 
   (define (init)
@@ -616,8 +645,11 @@
     (add-hook 'viewport-resized-hook update-workspace-hints)
     (add-hook 'viewport-moved-hook update-workspace-hints)
     (add-hook 'workarea-changed-hook update-workspace-hints)
+    (add-hook 'configure-notify-hook update-on-configure-notify)
 
-    (add-hook 'add-window-hook update-client-list-hints)
+    ;; Better not expose work in progress.  map-notify-hook gets
+    ;; called after this anyway.
+    ;;(add-hook 'add-window-hook update-client-list-hints)
     (add-hook 'destroy-notify-hook update-client-list-hints)
     (add-hook 'map-notify-hook update-client-list-hints)
     (add-hook 'unmap-notify-hook update-client-list-hints)
@@ -626,8 +658,9 @@
     (add-hook 'before-add-window-hook honour-client-state)
     (add-hook 'add-window-hook update-client-state)
     (call-after-state-changed '(sticky shaded maximized stacking
-				window-list-skip task-list-skip)
+                                       window-list-skip task-list-skip)
 			      update-client-state)
+    (call-after-state-changed 'sticky update-window-workspace-hints)
 
     (add-hook 'focus-in-hook update-focus-state)
     (add-hook 'focus-out-hook update-focus-state)
