@@ -35,6 +35,7 @@
 	     mark-window-as-dock
 	     window-in-cycle-p
 	     window-class
+	     window-pid
 	     warp-cursor-to-window
 	     activate-window
 	     constrain-dimension-to-hints
@@ -62,6 +63,12 @@
     (open rep
 	  rep.system
 	  rep.regexp
+	  #|
+	    Don't add more sawfish.wm.* here unless you know what
+	    you're doing. Instead, embed 'require' in definition.
+	    It's because this file is read in early stage of Sawfish
+	    init.
+	  |#
 	  sawfish.wm.windows.subrs
 	  sawfish.wm.gaol
 	  sawfish.wm.custom
@@ -70,11 +77,6 @@
 	  sawfish.wm.commands)
 
 
-  (defcustom ignore-window-input-hint nil
-    "Give focus to windows even when they haven't asked for it."
-    :type boolean
-    :group focus)
-
   (defgroup warp "Warping" :group misc)
 
   (defcustom warp-to-window-offset (cons -1 -1)
@@ -147,9 +149,7 @@ Returns nil if no such window is found."
   (define (window-really-wants-input-p w)
     "Return nil if window W should never be focused."
     (and (not (window-get w 'never-focus))
-	 (or ignore-window-input-hint
-	     (window-get w 'ignore-window-input-hint)
-	     (window-wants-input-p w))))
+	 (window-wants-input-p w)))
 
   (define (window-transient-p w)
     "Return non-nil if WINDOW is a transient window. The returned value will
@@ -211,11 +211,27 @@ windows."
 	 (not (or (and (not ignore-cycle-skip) (window-get w 'cycle-skip))
 		  (desktop-window-p w)))))
 
-  (define (window-class w)
-    "Return the class that window W belongs to, as a string. Returns `nil' if W
-has no associated class."
+  (define (window-class w #!optional spec)
+    "If SPEC is nil, return the class that window W belongs to,
+as a string. Returns `nil' if W has no associated class.
+
+If SPEC is 'cons, then cons (Instance . Class) is returned.
+If SPEC is 'configurator, then the string of the form \"Instance/Class\"
+is returned."
     (let ((prop (get-x-text-property w 'WM_CLASS)))
-      (and prop (> (length prop) 1) (aref prop 1))))
+      (when (and prop (> (length prop) 1))
+	(case spec
+	  ((cons)
+	   (cons (aref prop 0) (aref prop 1)))
+	  ((configurator)
+	   (concat (aref prop 1) "/" (aref prop 0)))
+	  (t
+	   (and prop (> (length prop) 1) (aref prop 1)))))))
+
+  (define (window-pid win)
+    "Returns the window pid, or nil if not available."
+    (when (caddr (get-x-property win '_NET_WM_PID))
+      (aref (caddr (get-x-property win '_NET_WM_PID)) 0)))
 
   (define (get-window-wm-protocols w)
     "Return a list of symbols defining the X11 window manager protocols
@@ -436,6 +452,8 @@ string `uniquify-name-format' to generate unique names."
   (defvar select-window-cursor-shape 'crosshair)
 
   (define (select-window)
+    "Wait for a click, and returns the clicked window. Nil for the root
+window."
     (allow-events 'async-pointer)
     (when (grab-pointer nil select-window-cursor-shape)
       (unwind-protect
